# -*- coding: utf-8 -*-

# Copyright 2024 University of Liège
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import openmdao.api as om

class FlutterGroup(om.Group):
    """Generic aerostructural flutter group
    """
    def initialize(self):
        self.options.declare('struct', desc='structural solver', recordable=False)
        self.options.declare('xfer', desc='interpolator', recordable=False)
        self.options.declare('aero', desc='aerodynamic solver', recordable=False)
        self.options.declare('flutter', desc='flutter solver', recordable=False)

    def setup(self):
        self.add_subsystem('mesh_struct', self.options['struct'].get_mesh(), promotes=['x_struct0'])
        self.add_subsystem('mesh_aero', self.options['aero'].get_mesh(), promotes=['x_aero0'])
        self.add_subsystem('struct', self.options['struct'].get_solver(), promotes=['x_struct0', 'q_struct', 'M', 'K'])
        self.add_subsystem('xfer', self.options['xfer'].get_solver(), promotes=['x_struct0', 'q_struct', 'x_aero0', 'q_aero'])
        self.add_subsystem('aero', self.options['aero'].get_solver(), promotes=['x_aero0', 'q_aero', 'Q_re', 'Q_im'])
        self.add_subsystem('flutter', self.options['flutter'].get_solver(), promotes=['M', 'K', 'Q_re', 'Q_im', 'f', 'g', 'g_agg'])
